import type { Bucket } from "../bucket.js";
import type { ExecutableStep, ModifierStep, UnbatchedExecutableStep } from "../step";
import type { OperationPlan } from "./OperationPlan";
/** Non-branching, non-deferred */
export interface LayerPlanReasonRoot {
    type: "root";
}
/** Non-branching, non-deferred */
export interface LayerPlanReasonNullableField {
    type: "nullableBoundary";
    /**
     * Can be used such that the same LayerPlan can be used for two selection
     * sets for the same parent plan. In this case an additional output plan
     * would be added to the LayerPlan.
     *
     * Also needed for execution (see `executeBucket`).
     */
    parentStep: ExecutableStep;
}
/** Non-branching, non-deferred */
export interface LayerPlanReasonListItem {
    type: "listItem";
    /**
     * Can be used such that the same LayerPlan can be used for two lists for
     * the same parent plan. In this case an additional output plan would be
     * added to the LayerPlan.
     *
     * Also needed for execution (see `executeBucket`).
     */
    parentStep: ExecutableStep;
    /** If this listItem is to be streamed, the configuration for that streaming */
    stream?: {
        initialCount: number;
        label?: string;
    };
}
/** Non-branching, deferred */
export interface LayerPlanReasonSubscription {
    type: "subscription";
}
/** Non-branching, deferred */
export interface LayerPlanReasonMutationField {
    type: "mutationField";
    mutationIndex: number;
}
/** Non-branching, deferred */
export interface LayerPlanReasonDefer {
    type: "defer";
    label?: string;
}
/** Branching, non-deferred */
export interface LayerPlanReasonPolymorphic {
    type: "polymorphic";
    typeNames: string[];
    /**
     * Needed for execution (see `executeBucket`).
     */
    parentStep: ExecutableStep;
    polymorphicPaths: Set<string>;
}
/** Non-branching, non-deferred */
export interface LayerPlanReasonSubroutine {
    type: "subroutine";
    parentStep: ExecutableStep;
}
export declare function isBranchingLayerPlan(layerPlan: LayerPlan<any>): boolean;
export declare function isDeferredLayerPlan(layerPlan: LayerPlan<any>): boolean;
export declare function isPolymorphicLayerPlan(layerPlan: LayerPlan<any>): boolean;
export type LayerPlanReason = LayerPlanReasonRoot | LayerPlanReasonNullableField | LayerPlanReasonListItem | LayerPlanReasonSubscription | LayerPlanReasonMutationField | LayerPlanReasonDefer | LayerPlanReasonPolymorphic | LayerPlanReasonSubroutine;
export type HasParent<A extends LayerPlanReason> = A extends any ? A extends {
    parentStep: ExecutableStep;
} ? A : never : never;
export type LayerPlanReasonsWithParentStep = HasParent<LayerPlanReason>;
/** @internal */
export interface LayerPlanPhase {
    /**
     * If true, we should check before the layer plan executes to see if the
     * execution has already timed out.
     *
     * @see {@link RequestTools.stopTime}
     */
    checkTimeout: boolean;
    /**
     * A list of steps that can be ran in parallel at this point, since all
     * their previous dependencies have already been satisfied.
     */
    normalSteps: Array<{
        step: ExecutableStep;
    }> | undefined;
    /**
     * A list of 'isSyncAndSafe' steps with unbatchedExecute methods that can be
     * ran once the `normalSteps` have completed; they must only depend on steps
     * that have already been executed before them (including previous
     * unbatchedSyncAndSafeSteps in the same list).
     */
    unbatchedSyncAndSafeSteps: Array<{
        step: UnbatchedExecutableStep;
        /**
         * Store the result of the step here if you want - useful to avoid lookups
         * and when there's no storage. HIGHLY VOLATILE, will not survive a tick!
         */
        scratchpad: any;
    }> | undefined;
    /**
     * Optimization - a digest of all steps in normalSteps and unbatchedSyncAndSafeSteps
     *
     * @internal
     */
    _allSteps: ExecutableStep[];
}
/**
 * A LayerPlan represents (via "reason") either the root (root), when something
 * happens at a later time (mutationField, defer), when plurality changes
 * (list, stream, subscription, polymorphic), or when a subprocess needs to be
 * computed (subroutine).
 *
 * Layer plans belong to an operation plan.
 *
 * Every layer plan (except for the root layer plan) has exactly one parent
 * layer plan.
 *
 * Every layer plan is caused by a parent step.
 *
 * The LayerPlan of a step influences:
 *
 * 1. how steps are deduplicated
 * 2. the order in which the steps are executed
 * 3. where the result of executing the step is stored
 * 4. when the step execution cache is allowed to be GC'd
 *
 * NOTE: `__ListTransformStep`'s effectively have a temporary bucket inside
 * them (built on the `__Item`) that's thrown away once the transform is
 * complete.
 *
 */
export declare class LayerPlan<TReason extends LayerPlanReason = LayerPlanReason> {
    readonly operationPlan: OperationPlan;
    parentLayerPlan: LayerPlan | null;
    readonly reason: TReason;
    id: number;
    /**
     * Every layer plan has a "root step" that shapes the value the layer
     * returns. Note that this step may be dependent on other steps included in
     * the LayerPlan, or could be provided externally.
     *
     * The root step is different for different layer step reasons:
     *
     * - root: the `operationPlan.rootValue`
     * - listItem: the `__ItemStep`
     * - stream: also the `__ItemStep`
     * - subscription: also the `__ItemStep`
     * - mutationField: the result plan of the mutation field
     * - defer: the parent layer's rootStep (defer always results in an object, unless an error occurs)
     * - polymorphic: the plan for the particular type
     * - subroutine: the result (returned) plan of the subroutine
     *
     * @internal
     */
    readonly rootStep: ExecutableStep | null;
    /**
     * Which plans the results for which are available in a parent bucket need to
     * be "copied across" to this bucket because plans in this bucket still
     * reference them?
     *
     * @internal
     */
    copyStepIds: number[];
    /** @internal */
    children: LayerPlan[];
    /** @internal */
    steps: ExecutableStep[];
    /** @internal */
    pendingSteps: ExecutableStep[];
    /**
     * Describes the order in which the steps within this LayerPlan are executed.
     *
     * Special attention must be paid to steps that have side effects.
     *
     * @internal
     */
    phases: Array<LayerPlanPhase>;
    /**
     * The list of layerPlans that steps added to this LayerPlan may depend upon.
     * Note this includes self, so it has one more entry than `depth`.
     *
     * ```
     * this.ancestry[this.depth] === this;
     * ```
     *
     * @internal
     */
    ancestry: LayerPlan[];
    /** How "deep" this layer plan is (how many ancestors it has). The root layer plan has a depth of 0. */
    depth: number;
    /** The depth at which a "defer boundary" occurs (OperationPlan.getPeers cannot pass a defer boundary), or 0. */
    deferBoundaryDepth: number;
    /**
     * An optimization for OperationPlan.getPeers; this tracks the steps in this
     * layer plan, grouped by their step class.
     */
    stepsByConstructor: Map<Function, Set<ExecutableStep>>;
    constructor(operationPlan: OperationPlan, parentLayerPlan: LayerPlan | null, reason: TReason);
    toString(): string;
    print(depth?: number): string;
    setRootStep($root: ExecutableStep): void;
    /** @internal Use plan.getStep(id) instead. */
    getStep(id: number, requestingStep: ExecutableStep): ExecutableStep;
    /** @internal */
    _addStep(step: ExecutableStep): number;
    /** @internal */
    _addModifierStep(step: ModifierStep<any>): string;
    finalize(): void;
    newBucket(parentBucket: Bucket): Bucket | null;
}
//# sourceMappingURL=LayerPlan.d.ts.map