import type { FragmentDefinitionNode, GraphQLFieldMap, GraphQLObjectType, GraphQLSchema, GraphQLUnionType, OperationDefinitionNode } from "graphql";
import type { Constraint } from "../constraints.js";
import type { SelectionSetDigest } from "../graphqlCollectFields.js";
import type { GrafastPlanJSON, ModifierStep } from "../index.js";
import { __TrackedValueStep, __ValueStep, ExecutableStep } from "../index.js";
import { $$timeout, $$ts } from "../interfaces.js";
import type { LayerPlanReasonSubroutine } from "./LayerPlan.js";
import { LayerPlan } from "./LayerPlan.js";
import { OutputPlan } from "./OutputPlan.js";
import { StepTracker } from "./StepTracker.js";
export declare const POLYMORPHIC_ROOT_PATH: null;
export declare const POLYMORPHIC_ROOT_PATHS: ReadonlySet<string> | null;
/** @internal */
export type OperationPlanPhase = "init" | "plan" | "validate" | "optimize" | "finalize" | "ready";
export interface MetaByMetaKey {
    [metaKey: string | number | symbol]: Record<string, any>;
}
export declare class OperationPlan {
    readonly schema: GraphQLSchema;
    readonly operation: OperationDefinitionNode;
    readonly fragments: {
        [fragmentName: string]: FragmentDefinitionNode;
    };
    readonly variableValues: {
        [key: string]: any;
    };
    readonly context: {
        [key: string]: any;
    };
    readonly rootValue: any;
    private readonly planningTimeout;
    readonly [$$timeout]: undefined;
    readonly [$$ts]: undefined;
    readonly queryType: GraphQLObjectType;
    readonly mutationType: GraphQLObjectType | null;
    readonly subscriptionType: GraphQLObjectType | null;
    readonly unionsContainingObjectType: {
        [objectTypeName: string]: ReadonlyArray<GraphQLUnionType>;
    };
    private operationType;
    /**
     * What state is the OpPlan in?
     *
     * 1. init
     * 2. plan
     * 3. validate
     * 5. optimize
     * 6. finalize
     * 7. ready
     *
     * Once in 'ready' state we can execute the plan.
     *
     * @internal
     */
    phase: OperationPlanPhase;
    /**
     * Gets updated as we work our way through the plan, useful for making errors more helpful.
     */
    loc: string[] | null;
    /** @internal */
    rootLayerPlan: LayerPlan;
    /**
     * Assigned during OperationPlan.planOperation(), guaranteed to exist after
     * initialization.
     *
     * @internal
     */
    rootOutputPlan: OutputPlan;
    private modifierStepCount;
    private modifierDepthCount;
    private modifierSteps;
    /** @internal */
    readonly stepTracker: StepTracker;
    private maxDeduplicatedStepId;
    private maxValidatedStepId;
    /** Constraints based on evaluating variables. @internal */
    readonly variableValuesConstraints: Constraint[];
    /** Stores the actual variableValues. @internal */
    readonly variableValuesStep: __ValueStep<{
        [key: string]: any;
    }>;
    /** A step for accessing variableValues in a tracked manner (allowing eval). @internal */
    readonly trackedVariableValuesStep: __TrackedValueStep<{
        [key: string]: any;
    }>;
    /** Constraints based on evaluating context. @internal */
    readonly contextConstraints: Constraint[];
    /** Stores the actual value of the context. @internal */
    readonly contextStep: __ValueStep<Grafast.Context>;
    /** Allows accessing context in a tracked manner (allowing eval). @internal */
    readonly trackedContextStep: __TrackedValueStep<{
        [key: string]: any;
    }>;
    /** Constraints based on evaluating rootValue. @internal */
    readonly rootValueConstraints: Constraint[];
    /** Stores the actual value of rootValue. @internal */
    readonly rootValueStep: __ValueStep<any>;
    /** Allows accessing rootValue in a tracked manner (allowing eval). @internal */
    readonly trackedRootValueStep: __TrackedValueStep<any>;
    /** @internal */
    makeMetaByMetaKey: () => MetaByMetaKey;
    /**
     * @internal
     */
    readonly itemStepIdByListStepId: {
        [listStepId: number]: number | undefined;
    };
    /**
     * If true, then this operation doesn't use (custom) resolvers.
     */
    pure: boolean;
    private startTime;
    private previousLap;
    private laps;
    private optimizeMeta;
    private scalarPlanInfo;
    constructor(schema: GraphQLSchema, operation: OperationDefinitionNode, fragments: {
        [fragmentName: string]: FragmentDefinitionNode;
    }, variableValues: {
        [key: string]: any;
    }, context: {
        [key: string]: any;
    }, rootValue: any, planningTimeout: number | null);
    private lap;
    private checkTimeout;
    /**
     * Called by the LayerPlan's constructor when it wants to get a new id to use.
     *
     * @internal
     */
    addLayerPlan(layerPlan: LayerPlan): number;
    /**
     * Adds a plan to the known steps and returns the number to use as the plan
     * id. ONLY to be used from Step, user code should never call this directly.
     *
     * @internal
     */
    _addStep(plan: ExecutableStep): number;
    /**
     * Adds a plan to the known steps and returns the number to use as the plan
     * id. ONLY to be used from Step, user code should never call this directly.
     *
     * @internal
     */
    _addModifierStep(step: ModifierStep<any>): string;
    /** @internal Use plan.getStep(id) instead. */
    getStep: (id: number, requestingStep: ExecutableStep) => ExecutableStep;
    /**
     * Get a plan without specifying who requested it; this disables all the
     * caller checks. Only intended to be called from internal code.
     *
     * @internal
     */
    dangerouslyGetStep(id: number): ExecutableStep;
    private planOperation;
    /**
     * Plans a GraphQL query operation.
     */
    private planQuery;
    /**
     * Implements the `PlanOpPlanMutation` algorithm.
     */
    private planMutation;
    /**
     * Implements the `PlanOpPlanSubscription` algorithm.
     */
    private planSubscription;
    /**
     * Gets the item plan for a given parent list plan - this ensures we only
     * create one item plan per parent plan.
     */
    private itemStepForListStep;
    processGroupedFieldSet(outputPlan: OutputPlan, path: readonly string[], polymorphicPath: string | null, polymorphicPaths: ReadonlySet<string> | null, parentStep: ExecutableStep, objectType: GraphQLObjectType, objectTypeFields: GraphQLFieldMap<any, any>, isMutation: boolean, groupedFieldSet: SelectionSetDigest): void;
    /**
     *
     * @param outputPlan - The output plan that this selection set is being added to
     * @param path - The path within the outputPlan that we're adding stuff (only for root/object OutputPlans)
     * @param parentStep - The step that represents the selection set root
     * @param objectType - The object type that this selection set is being evaluated for (note polymorphic selection should already have been handled by this point)
     * @param selections - The GraphQL selections (fields, fragment spreads, inline fragments) to evaluate
     * @param isMutation - If true this selection set should be executed serially rather than in parallel (each field gets its own LayerPlan)
     */
    private planSelectionSet;
    private planIntoOutputPlan;
    private polymorphicLayerPlanByPathByLayerPlan;
    private getPolymorphicLayerPlan;
    private planField;
    /**
     * A replacement for GraphQL's
     * `CoerceArgumentValues` that factors in tracked variables.
     *
     * @see https://spec.graphql.org/draft/#CoerceArgumentValues()
     */
    private getTrackedArguments;
    withModifiers<T>(cb: () => T): T;
    /**
     * Sets up tracking for the given value (variableValues, context, rootValue).
     */
    private track;
    /**
     * Checks that no step has a property on it whose value is another step. It
     * should addDependency instead.
     */
    private validateSteps;
    private replaceStep;
    private processStep;
    /**
     * Process the given steps, either dependencies first (root to leaf) or
     * dependents first (leaves to root).
     *
     * @internal
     */
    processSteps(actionDescription: string, order: "dependents-first" | "dependencies-first", isReadonly: boolean, callback: (plan: ExecutableStep) => ExecutableStep): void;
    /**
     * Peers are steps of the same type (but not the same step!) that are in
     * compatible layers and have the same dependencies. Peers must not have side
     * effects. A step is not its own peer.
     */
    private getPeers;
    private isImmoveable;
    /**
     * Attempts to hoist the step into a higher layerPlan to maximize
     * deduplication.
     */
    private hoistStep;
    /**
     * Attempts to push the step into the lowest layerPlan to minimize the need
     * for copying between layer plans.
     */
    private pushDown;
    private _deduplicateInnerLogic;
    private deduplicateStep;
    private deduplicateStepsProcess;
    /**
     * Gives us a chance to replace nearly-duplicate plans with other existing
     * plans (and adding the necessary transforms); this means that by the time
     * we come to optimize the plan tree should already be simpler. For example
     * if you have two plans at the same level that both request row data from
     * the same database table with the same identifiers, `WHERE`, `LIMIT`,
     * `OFFSET` and `ORDER BY`, but different `SELECT`s we could merge the two
     * plans together by replacing the latter with the former and having the
     * former SELECT additional fields, then transform the results back to what
     * our child plans would be expecting.
     */
    private deduplicateSteps;
    private hoistAndDeduplicate;
    private hoistSteps;
    private pushDownSteps;
    private getStepOptionsForStep;
    /**
     * Calls the 'optimize' method on a plan, which may cause the plan to
     * communicate with its (deep) dependencies, and even to replace itself with
     * a different plan.
     */
    private optimizeStep;
    /**
     * Note that we work through dependents first so we can make sure that we
     * know all our dependent's needs before we optimise ourself.
     */
    private optimizeSteps;
    /** Finalizes each step */
    private finalizeSteps;
    private finalizeLayerPlans;
    /** Optimizes each output plan */
    private optimizeOutputPlans;
    /** Finalizes each output plan */
    private finalizeOutputPlans;
    private walkOutputPlans;
    generatePlanJSON(): GrafastPlanJSON;
    finishSubroutine(subroutineStep: ExecutableStep, layerPlan: LayerPlan<LayerPlanReasonSubroutine>): void;
    /**
     * HIGHLY EXPERIMENTAL!
     *
     * @internal
     */
    deleteLayerPlan(layerPlan: LayerPlan): void;
    getStepsByMetaKey(metaKey: string | number | symbol): ExecutableStep[];
    getStepsByStepClass<TClass extends ExecutableStep>(klass: {
        new (...args: any[]): TClass;
    }): TClass[];
}
//# sourceMappingURL=OperationPlan.d.ts.map